import json
import geopandas as gpd
import pandas as pd
from shapely.geometry import Polygon


def get_relation(self, body_of_water: str):
    # Load GeoJSON data using geopandas
    geo_data = gpd.read_file("server/map/mjosa.geojson")

    # Filter only polygons, exclude points and other feature types
    polygon_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']

    # Extract coordinates from polygons and create polygon objects
    polygons = [Polygon(polygon.exterior) for polygon in polygon_data['geometry']]

    if len(polygons) <= 1:
        print("Failed to convert to polygons")

    # Divide relation into tiles and append tiles to relation object
    tiles = divide_relation(polygons)
    # polygon_data = pd.concat([polygon_data, gpd.GeoDataFrame(tiles)], ignore_index=True)

    # Transform geojson to json -> syncfusion_flutter_maps lib requires json
    geojson_dict = json.loads(polygon_data.to_json())
    response_json = json.dumps(geojson_dict)

    # Set headers
    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write coordinates to response object
    self.wfile.write(response_json.encode('utf-8'))


def divide_relation(polygons):
    # Define tile size
    tile_size = 0.01
    id = 1
    tiles = []

    for polygon in polygons:
        x_min, y_min, x_max, y_max = polygon.bounds
        rows = int((y_max - y_min) / tile_size)
        cols = int((x_max - x_min) / tile_size)

        if rows == 0 or cols == 0:  # Skip polygons that are smaller than the division size
            continue

        for row in range(rows):
            for col in range(cols):
                tile_bbox = Polygon([ # Calculate coordinate for current place in column and row
                    (x_min + col * tile_size, y_min + row * tile_size),
                    (x_min + (col + 1) * tile_size, y_min + row * tile_size),
                    (x_min + (col + 1) * tile_size, y_min + (row + 1) * tile_size),
                    (x_min + col * tile_size, y_min + (row + 1) * tile_size)
                ])
                tiles.append({"SubDivID": id, "polygon": tile_bbox})
                id += 1

    if len(tiles) <= 1:
        print("Failed to divide polygons into tiles")

    # Format each tile object with coordinate, type, and subdivision id tags
    tiles_json = []
    for tile in tiles:
        coordinates = list(tile["polygon"].exterior.coords)
        polygon_json = {
            "type": "Polygon",
            "coordinates": [[(coord[0], coord[1]) for coord in coordinates]],
            "SubDivID": tile["SubDivID"]
        }
        tiles_json.append(polygon_json)

    # Return all tiles for current polygon
    return tiles_json
